from freeldep.templates.core import CoreDeployerTemplate  # noqa
from freeldep.templates.initialize import InitializeDeployerTemplate  # noqa
from freeldep.templates.project import ProjectTemplate  # noqa
from freeldep.templates.repository import DeployerRepositoryTemplate  # noqa
from freeldep.templates.service import ServiceDeployerTemplate  # noqa
from freeldep.templates.subscription import SubscriptionDeployerTemplate  # noqa



from freeldep.cloud.aws.status import StackStatus


def test_success():
    assert StackStatus.success("CREATE_COMPLETE") is True
    assert StackStatus.success("UPDATE_COMPLETE") is True
    assert StackStatus.success("DELETE_COMPLETE") is True
    assert StackStatus.success("CREATE_FAILED") is False


def test_failed():
    assert StackStatus.failed("DELETE_FAILED") is True
    assert StackStatus.failed("CREATE_FAILED") is True
    assert StackStatus.failed("ROLLBACK_COMPLETE") is True
    assert StackStatus.failed("UPDATE_ROLLBACK_COMPLETE") is True
    assert StackStatus.failed("DELETE_COMPLETE") is False



import json
import os.path
import re
import subprocess
from os import listdir
from os.path import isfile
from os.path import join
from pathlib import Path

import yaml


def valid_expr(name, val, regex):
    if not bool(re.match(regex, val)):
        raise ValueError(f"{name} must match this format {regex}")


def shorten_expr(val, max_len):
    return val if len(val) < max_len else val[:max_len]


def mkdir(directory):
    Path(os.path.expanduser(directory)).mkdir(parents=True, exist_ok=True)


def file_exists(file_path):
    return os.path.exists(os.path.expanduser(file_path))


def isdir(directory):
    return os.path.isdir(os.path.expanduser(directory))


def list_files(directory):
    directory = os.path.expanduser(directory)
    return [f for f in listdir(directory) if isfile(join(directory, f))]


def save_json(filepath, event, indent=None):
    with open(os.path.expanduser(filepath), "w+") as f:
        json.dump(event, f, indent=indent)


def read_json(filepath):
    with open(os.path.expanduser(filepath), "r") as f:
        return json.load(f)


def read_yaml(filepath):
    with open(os.path.expanduser(filepath), "r") as f:
        return yaml.safe_load(f)


def save_deployer(deployer, config):
    """Save the deployer configuration locally"""
    home_folder = config.get("cli", "home_folder", "~/.freeldep")
    mkdir(home_folder)
    save_json(f"{home_folder}/{deployer['name']}.json", deployer)


def load_deployer(config, deployer):
    home_folder = config.get("cli", "home_folder", "~/.freeldep")
    config_file = home_folder + "/" + deployer + ".json"
    if not file_exists(config_file):
        return None
    return read_json(config_file)


def delete_deployer(config, deployer):
    home_folder = config.get("cli", "home_folder", "~/.freeldep")
    config_file = home_folder + "/" + deployer + ".json"
    if file_exists(config_file):
        os.remove(os.path.expanduser(config_file))


def execute_script(script):
    try:
        subprocess.check_call(f"/bin/bash {script}".split(" "))
        return True
    except subprocess.CalledProcessError:
        return False


def get_workflow_id(workflow_name, config):
    if config.get("aws", "account", None) is not None:
        region = config.get("aws", "region", "ap-southeast-1")
        account = config.get("aws", "account", None)
        return f"arn:aws:states:{region}:{account}:stateMachine:{workflow_name}"



import json

import click

from freeldep.modules.utils import file_exists
from freeldep.modules.utils import isdir
from freeldep.modules.utils import list_files
from freeldep.modules.utils import read_json


@click.group()
@click.pass_obj
def show(ctx):
    """show Freeldep variables"""
    pass


@show.command()
@click.argument("name")
@click.pass_context
def config(ctx, name):
    """Display resources used by a deployer"""
    home_folder = ctx.obj.config.get("cli", "home_folder", "~/.freeldep")
    config_file = home_folder + "/" + name + ".json"
    if file_exists(config_file):
        config = read_json(config_file)
        click.echo(json.dumps(config, indent=4))
    else:
        raise click.BadParameter(f"This deployer {name} does not exist")


@show.command()
@click.pass_context
def configs(ctx):
    """List all deployers already initialized"""
    home_folder = ctx.obj.config.get("cli", "home_folder", "~/.freeldep")
    if isdir(home_folder):
        config_files = list_files(home_folder)
        click.echo("\n".join([x.split(".")[0] for x in config_files]))




try:
    import importlib.resources as pkg_resources
except ImportError:
    # PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

from templates import aws
from templates import gcp



class InitializeDeployerTemplate:
    """Initialize a deployer with an artifact bucket and a registry table"""

    @classmethod
    def name(cls, deployer):
        return f"{deployer['name']}-deployer-initialization-stack"

    @classmethod
    def get(cls, cloud, deployer, config):
        if cloud == "AWS":
            return cls.aws(deployer, config)
        elif cloud == "GCP":
            return cls.gcp(deployer, config)
        raise NotImplementedError("Unknown cloud")

    @classmethod
    def aws(cls, deployer, config):
        return [
            {
                "aws": {
                    "region": config.get("aws", "region", None),
                    "account-id": config.get("aws", "account", None),
                    "deployment-role": config.get("aws", "deployment_role", ""),
                },
                #"location": "./templates/aws/deployer-initialize.yaml",
                "location": pkg_resources.open_text(aws, 'deployer-initialize.yaml'),
                "template": {
                    "name": cls.name(deployer),
                    "parameters": {
                        "artifact-bucket": deployer["artifact"],
                        "registry-table": deployer["registry"],
                    },
                },
                "functions": [],
            }
        ]

    @classmethod
    def gcp(cls, deployer, config):
        return [
            {
                "gcp": {
                    "zone": config.get("gcp", "zone", None),
                    "account-id": config.get("gcp", "account", None),
                    "project": config.get("gcp", "project", ""),
                },
                #"location": "./templates/gcp/deployer-initialize.yaml",
                "location": pkg_resources.open_text(gcp, 'deployer-initialize.yaml'),
                "template": {
                    "name": cls.name(deployer),
                    "parameters": {
                        "artifact-bucket": deployer["artifact"],
                        "bucket-location": config.get("gcp", "zone", None),
                    },
                },
                "functions": [],
                "commands": [],
            }
        ]



import json
import re
from random import randint

import click

from freeldep.cloud import Compiler
from freeldep.modules.utils import load_deployer
from freeldep.modules.utils import save_deployer
from freeldep.modules.utils import shorten_expr
from freeldep.modules.utils import valid_expr
from freeldep.templates import DeployerRepositoryTemplate
from freeldep.templates import InitializeDeployerTemplate
from freeldep.templates import ProjectTemplate
from freeldep.templates import SubscriptionDeployerTemplate


@click.group()
@click.pass_obj
def create(ctx):
    """Create new deployer"""
    pass


@create.command()
@click.argument("name")
@click.option(
    "--cloud",
    required=True,
    type=click.Choice(["AWS", "GCP"], case_sensitive=False),
    help="AWS or GCP cloud deployment",
)
@click.option(
    "--artifact-bucket",
    default=None,
    help="A bucket to store deployment artifacts will be automatically created using your"
    "deployer name. Use this option to overwrite default name.",
)
@click.option(
    "--registry-table",
    default=None,
    help="A registry table will be created to monitor deployment. Use this option to "
    "overwrite default name. Available for AWS only",
)
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def deployer(
    ctx, name, cloud, artifact_bucket, registry_table, wait, dryrun, output_location
):
    """Create artifact bucket and registry table. Initialize your deployer"""
    ctx.obj.require_cloud_access()
    cloud = cloud.upper()

    name = validate_name(name)
    artifact_bucket = validate_bucket(name, artifact_bucket)
    registry_table = validate_registry(name, registry_table) if cloud == "AWS" else None

    deployer = {
        "name": name,
        "cloud": cloud,
        "artifact": artifact_bucket,
        "registry": registry_table,
    }
    click.echo(json.dumps(deployer))

    config = ctx.obj.config
    Compiler.build_template(
        deployer,
        InitializeDeployerTemplate.get(cloud, deployer, config),
        cloud=cloud,
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )

    save_deployer(deployer, config)


@create.command()
@click.argument("name")
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option(
    "--emails",
    required=True,
    help="Comma delimited list of email addresses to send failed deployment notifications to",
)
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def subscription(ctx, name, deployer, emails, wait, dryrun, output_location):
    """Create a subscription and subscribe email addresses to it"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    if deployer.get("cloud", "").upper() == "GCP":
        raise RuntimeError("Subscriptions are not available for GCP")

    name = validate_name(name)
    emails = validate_emails(emails)
    if len(emails) == 0:
        raise click.BadParameter("There is no valid email address")

    Compiler.build_template(
        deployer,
        SubscriptionDeployerTemplate.get(name, emails, deployer, config),
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )

    if "subscriptions" not in deployer:
        deployer["subscriptions"] = []
    deployer["subscriptions"].append(f"{deployer['name']}-deployer-{name}")

    save_deployer(deployer, config)


@create.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def repository(ctx, deployer, wait, dryrun, output_location):
    """Create a repository with CI/CD to update core functionalities of your deployer"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")

    Compiler.build_template(
        deployer,
        DeployerRepositoryTemplate.get(deployer, config),
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )

    deployer["repository"] = f"{deployer['name']}-deployer"

    save_deployer(deployer, config)


@create.command()
@click.argument("name")
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option(
    "--branches", default="master,staging,dev", help="Branches triggering builds"
)
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def project(ctx, name, deployer, branches, wait, dryrun, output_location):
    """Create a new repository with CI/CD using a deployer"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")

    name = validate_name(name)
    branches = branches.strip()
    if "service-trigger" not in deployer or "service-role" not in deployer:
        raise click.UsageError("Create a deployer service to be able to create project")

    Compiler.build_template(
        deployer,
        ProjectTemplate.get(name, branches, deployer, config),
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )

    if "projects" not in deployer:
        deployer["projects"] = []
    deployer["projects"].append(name)

    save_deployer(deployer, config)


def validate_name(name):
    valid_expr("Name", name, "^[A-Za-z-]*$")
    return shorten_expr(name, 30).lower()


def validate_bucket(deployer_name, bucket):
    if bucket:
        valid_expr("Artifact bucket", bucket, "^[A-Za-z-]*$")
        return bucket
    else:
        return (
            deployer_name + "-deployer-artifact-bucket-" + str(randint(100000, 999999))
        )


def validate_registry(deployer_name, registry):
    if registry:
        valid_expr("Registry table", registry, "^[A-Za-z0-9-]*$")
        return shorten_expr(registry, 100).lower()
    else:
        return deployer_name + "-deployer-deployment-registry"


def validate_emails(emails):
    emails = emails.split(",")
    regex = r"^[a-z0-9]+[\._]?[a-z0-9]+[@]\w+[.]\w{2,6}$"
    out = []
    for email in emails:
        if re.search(regex, email):
            out.append(email)
    return out



import os
from subprocess import call

import pytest

from freeldep.templates import InitializeDeployerTemplate


def test_name(deployer):
    assert (
        InitializeDeployerTemplate.name(deployer)
        == "test-deployer-initialization-stack"
    )


def test_nocloud(deployer, config):
    testdeployer = deployer.copy()
    assert InitializeDeployerTemplate.get("AWS", testdeployer, config) is not None
    assert InitializeDeployerTemplate.get("GCP", testdeployer, config) is not None
    with pytest.raises(NotImplementedError):
        InitializeDeployerTemplate.get("sdsd", testdeployer, config)


def test_aws(deployer, config, configfile):
    template = InitializeDeployerTemplate.aws(deployer, config)

    assert len(template) == 1
    assert template[0]["aws"]["account-id"] == configfile["aws"].get("account")
    assert template[0]["aws"]["region"] == configfile["aws"].get("region")
    assert template[0]["aws"]["deployment-role"] == configfile["aws"].get(
        "deployment-role", ""
    )
    assert os.path.isfile(template[0]["location"])
    assert template[0]["template"]["name"] == InitializeDeployerTemplate.name(deployer)
    assert (
        template[0]["template"]["parameters"]["registry-table"] == deployer["registry"]
    )
    assert (
        template[0]["template"]["parameters"]["artifact-bucket"] == deployer["artifact"]
    )


def test_aws_template(deployer, config, configfile):
    template = InitializeDeployerTemplate.aws(deployer, config)
    assert 0 == call(f"cfn-lint {template[0]['location']}", shell=True)



import json
import os
import subprocess

import yaml
from google.cloud import storage  # pylint: disable=import-error

from freeldep.cloud.gcp.validate import DeploymentManagerValidation
from freeldep.cloud.stack import CloudStack


class DeploymentManagerTemplate(CloudStack):
    """Deployment Template: Read template, Inject parameters, Build functions and upload"""

    def __init__(self, config, deployer):
        super().__init__(config, deployer)
        DeploymentManagerValidation.validate_config(config)

    def _format_config(self):
        conf = {
            "GcpZone": self.config["gcp"]["zone"],
            "GcpProject": self.config["gcp"]["project"],
            "GcpAccount": self.config["gcp"]["account-id"],
        }
        for param, val in self.config["template"].items():
            conf["".join(x.title() for x in param.split("-"))] = val
        if "parameters" in self.config["template"]:
            for param, val in self.config["template"]["parameters"].items():
                conf["".join(x.title() for x in param.split("-"))] = val
        return conf

    def validate_template(self):
        """Validate template: How to do with GCP? #TODO"""
        if self.render_template is None:
            self.render_template()
        pass

    def deploy(self, wait=False, dryrun=False, output_folder=None):
        """Package Functions, deploy template"""
        self.package_functions(dryrun=dryrun, output_folder=output_folder)
        return self.deploy_template(dryrun=dryrun, output_folder=output_folder)

    def package_functions(self, dryrun=False, output_folder=None, gs_client=None):
        """Package Function into a zip file and upload to GS"""
        gs = storage.Client() if gs_client is None else gs_client
        bucket = gs.bucket(self.deployer["artifact"])
        for function in self.config["functions"]:
            filename = (
                function["filename"]
                if output_folder is None
                else output_folder + function["filename"]
            )
            self._compress_folder(function["location"], filename)
            key = self.config["template"][function["template-attribute"]]
            if not dryrun:
                self._upload_object(bucket, key, open(filename, "rb").read())
                os.remove(filename)

    def deploy_template(self, dryrun=False, output_folder=None):
        """Deploy template"""
        template, gcloud_commands = self._parse_template_commands()
        if template != "":
            template_name = self.config["template"]["name"]
            filename = (
                template_name
                if output_folder is None
                else output_folder + template_name
            )
            self._write_txt(template, filename + ".yaml")
            if dryrun:
                return
            status = self._get_deployment_status(template_name)
            action = "update" if status == "DONE" else "create"
            if not self._gcp_deploy(action, template_name, filename + ".yaml"):
                return False
        if len(gcloud_commands) > 0:
            for com in gcloud_commands:
                if not self._gcp_execute(com):
                    return False
        return True

    def cleanup(self, wait=True, dryrun=False, output_folder=None):
        """Delete existing stack"""
        template_name = self.config["template"]["name"]
        if self._get_deployment_status(template_name) != "DONE":
            return False, "Stack does not exist"
        if dryrun:
            return True
        return self._gcp_cleanup(template_name)

    def trigger_deployment(self, dryrun=False, output_folder=None):
        action = self.config.get("action", "create_or_update_stack").upper()
        if action == "DELETE_STACK":
            self.cleanup(dryrun=dryrun, output_folder=output_folder)
        else:
            success = self.deploy(self, dryrun=dryrun, output_folder=output_folder)
            if not success:
                if self.config.get("cleanup", False):
                    self.cleanup(dryrun=dryrun, output_folder=output_folder)
                raise RuntimeError("Deployment failed")

    def _parse_template_commands(self):
        parsed_template = yaml.safe_load(self.rendered_template)
        template = parsed_template.get("template", "")
        if template != "":
            template = yaml.dump(template)
        return template, parsed_template.get("gcloud", [])

    def _upload_object(self, bucket, key, content):
        """Upload object to GS"""
        bucket.blob(key).upload_from_string(content)

    def _get_deployment_status(self, template_name):
        """Get a deployment status
        :param deployment_name: name of the deployment stack
        """
        try:
            deployment = subprocess.check_output(
                [
                    "gcloud",
                    "deployment-manager",
                    "deployments",
                    "describe",
                    template_name,
                    "--format",
                    "json",
                ],
                stderr=subprocess.DEVNULL,
            )
            deployment = json.loads(deployment)
            return deployment["deployment"]["operation"]["status"]
        except subprocess.CalledProcessError:
            return "FAILED"

    def _gcp_deploy(self, command, template_name, template_path):
        try:
            subprocess.check_output(
                [
                    "gcloud",
                    "deployment-manager",
                    "deployments",
                    command,
                    template_name,
                    "--config",
                    template_path,
                ]
            )
            return True
        except subprocess.CalledProcessError:
            return False

    def _gcp_cleanup(self, template_name):
        """Delete a template
        :param template_name: name of the deployment
        """
        try:
            subprocess.check_output(
                [
                    "gcloud",
                    "deployment-manager",
                    "deployments",
                    "delete",
                    template_name,
                    "--quiet",
                ]
            )
            return True
        except subprocess.CalledProcessError:
            return False

    def _gcp_execute(self, command):
        """Get a deployment status
        :param deployment_name: name of the deployment stack
        """
        try:
            subprocess.check_output(command.split(" "))
            return True
        except subprocess.CalledProcessError:
            return False



import pytest

from freeldep.modules.deploy import deploy_project


def test_deploy_project():
    with pytest.raises(FileNotFoundError):
        deploy_project("test", "test", "test.yaml")
    with pytest.raises(RuntimeError):
        deploy_project("test", "test", "tests/data/stack.yaml", "tests/data/package.sh")



import os
import subprocess


class CodebuildHelper:

    BRANCH_EXTRACT_SCRIPT = """
        export CODEBUILD_GIT_BRANCH=`git symbolic-ref HEAD --short 2>/dev/null`
        if [ "$CODEBUILD_GIT_BRANCH" == "" ] ; then
            CODEBUILD_GIT_BRANCH=`git branch -a --contains HEAD | sed -n 2p | awk '{ printf $1 }'`
            export CODEBUILD_GIT_BRANCH=${CODEBUILD_GIT_BRANCH#remotes/origin/}
        fi
        echo $CODEBUILD_GIT_BRANCH;
    """

    @classmethod
    def is_codebuild(cls):
        return os.environ.get("CODEBUILD_SOURCE_REPO_URL", "") != ""

    @classmethod
    def get_project(cls):
        return os.environ["CODEBUILD_SOURCE_REPO_URL"].split("/")[-1]

    @classmethod
    def get_branch(cls):
        return subprocess.getoutput(cls.BRANCH_EXTRACT_SCRIPT)



class StackStatus:
    @classmethod
    def success(cls, status):
        return status in ["CREATE_COMPLETE", "UPDATE_COMPLETE", "DELETE_COMPLETE"]

    @classmethod
    def failed(cls, status):
        return status in [
            "DELETE_FAILED",
            "CREATE_FAILED",
            "ROLLBACK_COMPLETE",
            "UPDATE_ROLLBACK_COMPLETE",
        ]



import os

import pytest

from freeldep.templates import SubscriptionDeployerTemplate


def test_name(deployer):
    assert (
        SubscriptionDeployerTemplate.name(deployer, "testsub")
        == "test-deployer-testsub-subscription"
    )


def test_nocloud(deployer, config):
    subscriptions = ["sdfsdf"]
    testdeployer = deployer.copy()
    testdeployer["cloud"] = "AWS"
    assert (
        SubscriptionDeployerTemplate.get("testsub", subscriptions, testdeployer, config)
        is not None
    )
    with pytest.raises(NotImplementedError):
        testdeployer["cloud"] = "GCP"
        SubscriptionDeployerTemplate.get("testsub", subscriptions, testdeployer, config)
    with pytest.raises(NotImplementedError):
        testdeployer["cloud"] = "sdfsdfsdfsdf"
        SubscriptionDeployerTemplate.get("testsub", subscriptions, testdeployer, config)


def test_aws(deployer, config, configfile):
    subscriptions = ["sdfsdf"]
    template = SubscriptionDeployerTemplate.aws(
        "testsub", subscriptions, deployer, config
    )

    assert len(template) == 1
    assert template[0]["aws"]["account-id"] == configfile["aws"].get("account")
    assert template[0]["aws"]["region"] == configfile["aws"].get("region")
    assert template[0]["aws"]["deployment-role"] == configfile["aws"].get(
        "deployment-role", ""
    )
    assert os.path.isfile(template[0]["location"])
    assert template[0]["template"]["name"] == SubscriptionDeployerTemplate.name(
        deployer, "testsub"
    )
    assert template[0]["template"]["parameters"]["subscription-name"] == "testsub"
    assert len(template[0]["template"]["subscriptions"]) == len(subscriptions)



import click

from freeldep.cloud import Compiler
from freeldep.modules.utils import delete_deployer
from freeldep.modules.utils import load_deployer
from freeldep.modules.utils import save_deployer
from freeldep.templates import CoreDeployerTemplate
from freeldep.templates import DeployerRepositoryTemplate
from freeldep.templates import ProjectTemplate
from freeldep.templates import ServiceDeployerTemplate
from freeldep.templates import SubscriptionDeployerTemplate


@click.group()
@click.pass_obj
def cleanup(ctx):
    """Delete CFN stacks"""
    pass


# PROJECT
# REPO


@cleanup.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=False, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.pass_context
def core(ctx, deployer, wait, dryrun):
    """Deploy core step function"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    Compiler.cleanup(
        deployer, CoreDeployerTemplate.get(deployer, config), wait=wait, dryrun=dryrun
    )
    del deployer["deployment-workflow"]
    save_deployer(deployer, config)


@cleanup.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=False, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.pass_context
def service(ctx, deployer, wait, dryrun):
    """Deploy core step function"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")

    Compiler.cleanup(
        deployer,
        ServiceDeployerTemplate.get(deployer, config),
        wait=wait,
        dryrun=dryrun,
    )
    del deployer["codebuild-service"]
    del deployer["service-trigger"]
    del deployer["service-role"]
    save_deployer(deployer, config)


@cleanup.command()
@click.argument("name")
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=False, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.pass_context
def subscription(ctx, name, deployer, wait, dryrun):
    """Deploy core step function"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    if "subscriptions" not in deployer or name not in deployer["subscriptions"]:
        raise click.BadParameter(f"Subscription {name} does not exists")
    Compiler.cleanup(
        deployer,
        SubscriptionDeployerTemplate.get(name, [], deployer, config),
        wait=wait,
        dryrun=dryrun,
    )
    deployer["subscriptions"] = [x for x in deployer["subscriptions"] if x != name]
    save_deployer(deployer, config)


@cleanup.command()
@click.argument("name")
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.pass_context
def project(ctx, name, deployer, wait, dryrun):
    """Remove the CICD integration from existing project. Does not delete the repository."""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    template = ProjectTemplate.get(name, "", deployer, config)
    template[0]["template"]["parameters"]["cleanup"] = "true"
    Compiler.cleanup(deployer, template, wait=wait, dryrun=dryrun)


@cleanup.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=True, help="Wait for cleanup completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.pass_context
def repository(ctx, deployer, wait, dryrun):
    """Remove the CICD integration of your deployer repository. Does not delete the repository"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    template = DeployerRepositoryTemplate.get(deployer, config)
    template[0]["template"]["parameters"]["cleanup"] = "true"
    Compiler.cleanup(deployer, template, wait=wait, dryrun=dryrun)
    del deployer["repository"]
    save_deployer(deployer, config)


@cleanup.command()
@click.argument("name")
@click.option(
    "--confirm/--no-confirm",
    default=False,
    help="Confirm you want to delete your configuration file",
)
@click.pass_context
def deployer(ctx, name, confirm):
    """Delete deployer configuration. Does not delete the repository"""
    config = ctx.obj.config
    deployer = load_deployer(config, name)
    if deployer is None:
        raise click.BadParameter(f"Deployer {name} does not exist")
    if confirm:
        delete_deployer(config, name)
    else:
        raise click.UsageError(
            "Add the flag --confirm to confirm you want to delete your deployer"
        )




try:
    import importlib.resources as pkg_resources
except ImportError:
    # PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

from templates import aws



class DeployerRepositoryTemplate:
    """Create a repository for continuous integration of deployer"""

    @classmethod
    def name(cls, deployer):
        return f"{deployer['name']}-deployer-repository-stack"

    @classmethod
    def get(cls, deployer, config):
        cloud = deployer.get("cloud", None)
        if cloud == "AWS":
            return cls.aws(deployer, config)
        raise NotImplementedError("Unknown cloud")

    @classmethod
    def aws(cls, deployer, config):
        return [
            {
                "aws": {
                    "region": config.get("aws", "region", "ap-southeast-1"),
                    "account-id": config.get("aws", "account", None),
                    "deployment-role": config.get("aws", "deployment_role", ""),
                },
                #"location": "./templates/aws/deployer-repository.yaml",
                "location": pkg_resources.open_text(aws, 'deployer-repository.yaml'),
                "template": {
                    "name": cls.name(deployer),
                    "parameters": {
                        "repository": f"{deployer['name']}-deployer",
                        "repository-codebuild": f"{deployer['name']}-deployer"
                        "-continuous-integration",
                        "artifact-bucket": deployer["artifact"],
                        "deployment-workflow": f"{deployer['name']}-deployer-core",
                    },
                },
                "functions": [],
            }
        ]



import os


class DeploymentManagerValidation:
    @classmethod
    def validate_config(cls, config):
        """Validate section of the template config"""
        if "gcp" not in config:
            raise KeyError("gcp is required in a stack definition")
        else:
            cls._validate_gcp_config(config["gcp"])

        if "location" not in config:
            raise KeyError("location is required in a stack definition")
        elif not cls._file_exists(os.path.expanduser(config["location"])):
            raise FileNotFoundError(f"Template {config['location']} does not exist")

        if "template" not in config:
            raise KeyError("template is required in a stack definition")
        else:
            cls._validate_template_config(config["template"])

        if "functions" in config:
            cls._validate_functions_config(config["functions"])

    @classmethod
    def _validate_gcp_config(cls, config):
        """Validate section of the template config"""
        if type(config) != dict:
            raise ValueError("gcp must be a dict")
        if "account-id" not in config:
            raise KeyError("gcp.account-id is required in template definition")
        if "project" not in config:
            raise KeyError("gcp.project is required in template definition")
        if "zone" not in config:
            raise KeyError("gcp.zone is required in template definition")

    @classmethod
    def _validate_template_config(cls, config):
        """Validate section of the template config"""
        if type(config) != dict:
            raise ValueError("template must be a dict")
        if "name" not in config:
            raise KeyError("template.name is required in stack definition")
        if type(config["name"]) != str:
            raise ValueError("template.name must be a string")
        if "parameters" in config and type(config["parameters"]) != dict:
            raise ValueError("template.parameters must be a dict")

    @classmethod
    def _validate_functions_config(cls, config):
        """Validate section of the stack config"""
        for function in config:
            if "name" not in function:
                raise KeyError("Function must have a name")
            if "location" not in function:
                raise KeyError("Function must have a location")
            if "template-attribute" not in function:
                raise KeyError("Function must have a template-attribute")
            if "bucket" not in function:
                raise KeyError("Function must have an artifact bucket location")
            if (
                type(function["name"]) != str
                or type(function["template-attribute"]) != str
                or type(function["bucket"]) != str
            ):
                raise ValueError(
                    "One of these parameters is not a string: name, template-attribute, bucket"
                )
            if not os.path.isdir(function["location"]):
                raise ValueError("Function package is not found")

    @classmethod
    def _file_exists(cls, file_path):
        """Check if a file exists"""
        return os.path.exists(os.path.expanduser(file_path))



import configparser
import os.path

import click


class ConfigParser:
    def __init__(self, config_name):
        self.config = None
        if os.environ.get("FREELDEP_CONFIG", ""):
            config_name = os.environ["FREELDEP_CONFIG"]
        elif os.environ.get("FREELDEP_HOME", ""):
            config_file = self._parse_home()
            if config_file:
                config_name = config_file
        if os.path.isfile(config_name):
            self.config = configparser.ConfigParser()
            self.config.read(config_name)
        elif not os.environ.get("CODEBUILD_SOURCE_REPO_URL", None):
            raise click.UsageError(
                "You must specify your deployment settings. Set FREELDEP_CONFIG, or FREELDEP_HOME"
                "or create a file ./config.ini"
            )

    def get(self, section, name, default=None):
        if self.config is None or section not in self.config.sections():
            return default
        return self.config[section].get(name, default)

    def _parse_home(self):
        folder = os.environ["FREELDEP_HOME"]
        if folder[-1] != "/":
            folder = folder + "/"
        filename = folder + "config.ini"
        if os.path.isfile(os.path.expanduser(filename)):
            return filename



import os

import click

from freeldep.cloud import Compiler
from freeldep.cloud.aws.build import CodebuildHelper
from freeldep.modules.utils import execute_script
from freeldep.modules.utils import file_exists
from freeldep.modules.utils import get_workflow_id
from freeldep.modules.utils import load_deployer
from freeldep.modules.utils import read_yaml
from freeldep.modules.utils import save_deployer
from freeldep.templates import CoreDeployerTemplate
from freeldep.templates import ServiceDeployerTemplate


@click.group()
@click.pass_obj
def deploy(ctx):
    """Deploy deployer"""
    pass


@deploy.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=True, help="Wait for completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def core(ctx, deployer, wait, dryrun, output_location):
    """Deploy core step function"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    Compiler.build_template(
        deployer,
        CoreDeployerTemplate.get(deployer, config),
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )
    deployer["deployment-workflow"] = f"{deployer['name']}-deployer-core"
    deployer["account"] = config.get("aws", "account", None)
    save_deployer(deployer, config)


@deploy.command()
@click.option("--deployer", required=True, help="Name of your deployer")
@click.option("--wait/--no-wait", default=True, help="Wait for completion")
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def service(ctx, deployer, wait, dryrun, output_location):
    """Create a service for other codecommit repositories"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    deployer = load_deployer(config, deployer)
    if deployer is None:
        raise click.BadParameter(f"Deployer {deployer} does not exist")
    Compiler.build_template(
        deployer,
        ServiceDeployerTemplate.get(deployer, config),
        wait=wait,
        dryrun=dryrun,
        output_folder=output_location,
    )
    deployer["codebuild-service"] = f"{deployer['name']}-deployer-service"
    deployer["service-trigger"] = f"{deployer['name']}-deployer-service-trigger"
    deployer["service-role"] = f"{deployer['name']}-deployer-service-role"
    save_deployer(deployer, config)


@deploy.command()
@click.option("--deployer", default=None, help="Name of your deployer")
@click.option("--project", default=None, help="Project name")
@click.option("--stack-file", required=True, help="Path of your templates to deploy")
@click.option(
    "--package-script",
    default="./bin/package.sh",
    help="Bash script to execute before deploying",
)
@click.option(
    "--dryrun/--no-dryrun",
    default=False,
    help="If enable, does not really create a resource",
)
@click.option(
    "--output-location",
    default=None,
    help="If dryrun enable, save the artifacts to this location",
)
@click.pass_context
def project(
    ctx, deployer, project, stack_file, package_script, dryrun, output_location
):
    """Deploy a project build"""
    ctx.obj.require_cloud_access()
    config = ctx.obj.config
    if not CodebuildHelper.is_codebuild():
        if deployer is None:
            raise click.UsageError("You must specify a deployer")
        deployer = load_deployer(config, deployer)
        if deployer is None:
            raise click.BadParameter(f"Deployer {deployer} does not exist")
        if "deployment-workflow" not in deployer:
            raise click.UsageError("Your deployer doesn't have a deployment workflow")
        deployer["deployment-workflow-id"] = get_workflow_id(
            deployer["deployment-workflow"], config
        )
        deployer["project"] = project
    else:
        deployer = {
            "artifact": os.environ["ARTIFACTS_BUCKET"],
            "deployment-workflow-id": os.environ["DEPLOYER_STATE_MACHINE_ARN"],
            "project": CodebuildHelper.get_project(),
        }
    deploy_project(
        deployer, project, stack_file, package_script, dryrun, output_location
    )


def deploy_project(
    deployer,
    project,
    stack_file,
    package_script="./bin/package.sh",
    dryrun=False,
    output_location=None,
):
    if not file_exists(stack_file):
        raise FileNotFoundError("No stack to deploy")
    stacks = read_yaml(stack_file)

    if file_exists(package_script) and not execute_script(package_script):
        raise RuntimeError("Packge script failed")

    Compiler.trigger_deployment(
        deployer, stacks, dryrun=dryrun, output_folder=output_location
    )



import os

import pytest

from freeldep.templates import CoreDeployerTemplate
from freeldep.templates import InitializeDeployerTemplate
from freeldep.templates import ServiceDeployerTemplate


def test_name(deployer):
    assert ServiceDeployerTemplate.name(deployer) == "test-deployer-service-stack"


def test_nocloud(deployer, config):
    testdeployer = deployer.copy()
    testdeployer["cloud"] = "AWS"
    assert ServiceDeployerTemplate.get(testdeployer, config) is not None
    with pytest.raises(NotImplementedError):
        testdeployer["cloud"] = "GCP"
        ServiceDeployerTemplate.get(testdeployer, config)
    with pytest.raises(NotImplementedError):
        testdeployer["cloud"] = "sdfsdfsdfsdf"
        ServiceDeployerTemplate.get(testdeployer, config)


def test_aws(deployer, config, configfile):
    template = ServiceDeployerTemplate.aws(deployer, config)
    ref_template = InitializeDeployerTemplate.aws(deployer, config)

    assert len(template) == 1
    assert template[0]["aws"]["account-id"] == configfile["aws"].get("account")
    assert template[0]["aws"]["region"] == configfile["aws"].get("region")
    assert template[0]["aws"]["deployment-role"] == configfile["aws"].get(
        "deployment-role", ""
    )
    assert os.path.isfile(template[0]["location"])
    assert template[0]["template"]["name"] == ServiceDeployerTemplate.name(deployer)
    assert (
        template[0]["template"]["parameters"]["artifact-bucket"]
        == ref_template[0]["template"]["parameters"]["artifact-bucket"]
    )
    assert (
        template[0]["template"]["parameters"]["deployment-workflow"]
        == CoreDeployerTemplate.aws(deployer, config)[0]["template"]["parameters"][
            "deployment-workflow"
        ]
    )
    assert (
        template[0]["template"]["lambda-code-key"]
        == f"packages/{deployer['name']}-deployer-service-stack/"
    )
    assert len(template[0]["functions"]) == 1
    assert os.path.isdir(template[0]["functions"][0]["location"])



import json
import logging

from freeldep.cloud.aws.stack import CfnStack
from freeldep.cloud.gcp.stack import DeploymentManagerTemplate

AWS_CLOUD = "AWS"
GCP_CLOUD = "GCP"


class Compiler:
    @classmethod
    def build_template(
        cls,
        deployer,
        templates,
        cloud=AWS_CLOUD,
        wait=True,
        dryrun=False,
        output_folder=None,
    ):

        for template in templates:
            stack = cls.get_stack(cloud, template, deployer)
            stack.read_template()
            stack.prepare_packages()
            stack.render_template()
            stack.validate_template()
            stack.deploy(wait=wait, dryrun=dryrun, output_folder=output_folder)

    @classmethod
    def cleanup(
        cls,
        deployer,
        templates,
        cloud=AWS_CLOUD,
        wait=True,
        dryrun=False,
        output_folder=None,
    ):
        for template in templates:
            logging.info(json.dumps(template, indent=4))
            stack = cls.get_stack(cloud, template, deployer)
            stack.cleanup(wait=wait, dryrun=dryrun, output_folder=output_folder)

    @classmethod
    def trigger_deployment(
        cls,
        deployer,
        templates,
        cloud=AWS_CLOUD,
        wait=True,
        dryrun=False,
        output_folder=None,
    ):
        for template in templates:
            logging.info(json.dumps(template, indent=4))
            stack = cls.get_stack(cloud, template, deployer)
            stack.read_template()
            stack.prepare_packages()
            stack.render_template()
            stack.validate_template()
            stack.trigger_deployment(dryrun=dryrun, output_folder=output_folder)

    @classmethod
    def get_stack(cls, cloud, template, deployer):
        if cloud == AWS_CLOUD:
            return CfnStack(template, deployer)
        if cloud == GCP_CLOUD:
            return DeploymentManagerTemplate(template, deployer)
        raise NotImplementedError("Cloud not found")



import pytest

from freeldep.modules.create import validate_bucket
from freeldep.modules.create import validate_emails
from freeldep.modules.create import validate_name
from freeldep.modules.create import validate_registry


def test_validate_name():
    assert validate_name("sdfsdfdfsdf") == "sdfsdfdfsdf"
    assert (
        validate_name("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")
        == "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    )
    with pytest.raises(ValueError):
        validate_name("sdfsdfd223!#fsdf") == "sdfsdfdfsdf"


def test_validate_bucket():
    assert validate_bucket("test", "testbucket") == "testbucket"
    with pytest.raises(ValueError):
        validate_bucket("test", "testbucket@")
    bucket = validate_bucket("test", None)
    assert bucket.startswith("test")
    assert bucket.startswith("test-deployer-artifact-bucket-")
    assert len(bucket.split("-")[-1]) == 6


def test_validate_registry():
    assert validate_registry("test", "testregistry") == "testregistry"
    assert (
        validate_registry(
            "test",
            "testregistrysssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss"
            "sssssssssssssssssssssssss",
        )
        == "testregistrysssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss"
        "sssssssssssssssssssss"
    )
    with pytest.raises(ValueError):
        validate_bucket("test", "testregistry@")
    registry = validate_registry("test", None)
    assert registry.startswith("test")


def test_validate_emails():
    assert validate_emails("test@gmail.com")[0] == "test@gmail.com"
    assert len(validate_emails("test@gmail.com,test2@gmail.com")) == 2
    assert len(validate_emails("test@gmail.com,test2")) == 1



import os

from freeldep.cloud.aws.build import CodebuildHelper


def test_is_codebuild():
    os.environ["CODEBUILD_SOURCE_REPO_URL"] = "test"
    assert CodebuildHelper.is_codebuild() is True
    os.environ["CODEBUILD_SOURCE_REPO_URL"] = ""
    assert CodebuildHelper.is_codebuild() is False


def test_get_project():
    os.environ[
        "CODEBUILD_SOURCE_REPO_URL"
    ] = "https://git-codecommit.ap-southeast-1.amazonaws.com/v1/repos/de-testproj"
    assert CodebuildHelper.get_project() == "de-testproj"
    os.environ["CODEBUILD_SOURCE_REPO_URL"] = ""
    assert CodebuildHelper.get_project() == ""


def test_get_branch():
    CodebuildHelper.BRANCH_EXTRACT_SCRIPT += "echo master;"
    assert CodebuildHelper.get_branch().split("\n")[-1] == "master"



import os
import subprocess
import uuid


class CfnStackValidation:
    @classmethod
    def validate_config(cls, config):
        """Validate section of the stack config"""
        if "aws" not in config:
            raise KeyError("aws is required in a stack definition")
        else:
            cls._validate_aws_config(config["aws"])

        if "location" not in config:
            raise KeyError("location is required in a stack definition")
        elif type(config['location']) == str and not cls._file_exists(os.path.expanduser(config["location"])):
            raise FileNotFoundError(f"Template {config['location']} does not exist")

        if "template" not in config:
            raise KeyError("template is required in a stack definition")
        else:
            cls._validate_template_config(config["template"])

        if "functions" in config:
            cls._validate_lambda_config(config["functions"])

    @classmethod
    def validate_stack(cls, template, tmp_location=f"/tmp/{uuid.uuid1().hex}.yaml"):
        """Validate CFN stack with CFNLINT"""
        with open(tmp_location, "w+") as f:
            f.write(template)
        try:
            subprocess.check_output(
                f"cfn-lint {tmp_location}".split(" "), stderr=subprocess.STDOUT
            )
        except subprocess.CalledProcessError:
            raise RuntimeError("Your CFN stack is not valid")
        os.remove(tmp_location)

    @classmethod
    def _validate_aws_config(cls, config):
        """Validate section of the stack config"""
        if type(config) != dict:
            raise ValueError("aws must be a dict")
        if "region" not in config:
            raise KeyError("aws.region is required in stack definition")
        if "account-id" not in config:
            raise KeyError("aws.account-id is required in stack definition")
        if type(config["region"]) != str:
            raise ValueError("aws.region must be a string")
        if (
            type(config["account-id"]) != str
            or len(str(config["account-id"])) != 12
            or len([x for x in config["account-id"] if not x.isdigit()])
        ):
            raise ValueError("aws.account-id must be a 12 digit string")

    @classmethod
    def _validate_template_config(cls, config):
        """Validate section of the stack config"""
        if type(config) != dict:
            raise ValueError("template must be a dict")
        if "name" not in config:
            raise KeyError("template.name is required in stack definition")
        if type(config["name"]) != str:
            raise ValueError("template.name must be a string")
        if "parameters" in config and type(config["parameters"]) != dict:
            raise ValueError("template.parameters must be a dict")

    @classmethod
    def _validate_lambda_config(cls, config):
        """Validate section of the stack config"""
        for lambd in config:
            if "name" not in lambd:
                raise KeyError("Lambdas must have a name")
            if "location" not in lambd:
                raise KeyError("Lambdas must have a location")
            if "template-attribute" not in lambd:
                raise KeyError("Lambdas must have a template-attribute")
            if "bucket" not in lambd:
                raise KeyError("Lambdas must have a artifact bucket location")
            if (
                type(lambd["name"]) != str
                or type(lambd["template-attribute"]) != str
                or type(lambd["bucket"]) != str
            ):
                raise ValueError(
                    "One of these parameters is not a string: name, template-attribute, bucket"
                )
            if not os.path.isdir(lambd["location"]):
                raise ValueError("Lambda package is not found")

    @classmethod
    def _file_exists(cls, file_path):
        """Check if a file exists"""
        return os.path.exists(os.path.expanduser(file_path))



import logging

import click

from freeldep.config import ConfigParser
from freeldep.modules import cleanup
from freeldep.modules import create
from freeldep.modules import deploy
from freeldep.modules import show
from freeldep.cloud.aws.build import CodebuildHelper

class Cli(object):
    def __init__(self, aws_profile=None, verbose=0, config_file="./cli/config.ini"):
        logging.basicConfig(
            format="%(asctime)s - %(message)s", level=self._get_logging_level(verbose)
        )
        self.config = ConfigParser(config_file)
        self.name = self.config.get("cli", "name", "Freeldep")
        self.version = self.config.get("cli", "version", "latest")
        self.description = self.config.get("cli", "description", "Serverless Infrastructure Deployment")
        self.aws_profile = aws_profile

    def require_cloud_access(self):
        if self.aws_profile is None and not CodebuildHelper.is_codebuild():
            raise RuntimeError("Please set AWS_PROFILE to deploy resources")

    def _get_logging_level(self, verbose):
        if verbose < 1:
            return logging.WARNING
        if verbose == 1:
            return logging.INFO
        return logging.DEBUG


@click.group()
@click.option("--aws-profile", envvar="AWS_PROFILE", default=None)
@click.option("-v", "--verbose", count=True, default=0)
@click.option("--config-file", envvar="FREELDEP_CONFIG", default="./cli/config.ini")
@click.pass_context
def cli(ctx, aws_profile, verbose, config_file):
    """Infrastructure as Code deployment CLI"""
    ctx.obj = Cli(aws_profile, verbose, config_file)


@cli.command()
@click.pass_context
def version(ctx):
    click.echo(f"{ctx.obj.name} CLI")
    click.echo(f"Version: {ctx.obj.version}")


cli.add_command(create.create)
cli.add_command(show.show)
cli.add_command(deploy.deploy)
cli.add_command(cleanup.cleanup)



import logging
import os

import pytest
from click.testing import CliRunner

from freeldep.main import Cli
from freeldep.main import cli


def test_cli():
    runner = CliRunner()
    result = runner.invoke(cli)
    assert result.exit_code == 0


def test_cli_logging():
    cli_obj = Cli(aws_profile=None, verbose=0, config_file="tests/data/config.ini")
    assert cli_obj._get_logging_level(0) == logging.WARNING
    assert cli_obj._get_logging_level(1) == logging.INFO
    assert cli_obj._get_logging_level(2) == logging.DEBUG
    assert cli_obj._get_logging_level(1029) == logging.DEBUG
    assert cli_obj._get_logging_level(-1029) == logging.WARNING


def test_cli_cloud():
    os.environ["AWS_PROFILE"] = ""
    os.environ["FREELDEP_CONFIG"] = "tests/data/config.ini"
    cli_obj = Cli()
    with pytest.raises(RuntimeError):
        cli_obj.require_cloud_access()
    cli_obj = Cli(aws_profile="default")
    cli_obj.require_cloud_access()


def test_version():
    runner = CliRunner()
    result = runner.invoke(cli, ["version"])
    assert result.exit_code == 0
    assert "CLI" in result.output and "Version" in result.output



import os


class CloudBuildHelper:
    @classmethod
    def is_cloudrun(cls):
        return (
            os.environ.get("PROJECT_ID", "") != ""
            and os.environ.get("BUILD_ID", "") != ""
        )

    @classmethod
    def get_project(cls):
        return os.environ["REPO_NAME"]

    @classmethod
    def get_branch(cls):
        return os.environ["BRANCH_NAME"]



#!/usr/bin/env python
from setuptools import find_packages
from setuptools import setup

PROJECT = "freeldep"
VERSION = "0.7.1"


try:
    long_description = open("README.md", "rt").read()
except IOError:
    long_description = ""

setup(
    name=PROJECT,
    version=VERSION,
    description="Serverless Infrastructure as code deployment",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Matthieu Blais",
    author_email="matthieu@freeldom.com",
    url="https://github.com/MatthieuBlais/freeldep",
    download_url="https://github.com/MatthieuBlais/freeldep.git",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "Environment :: Console",
    ],
    platforms=["Any"],
    scripts=[],
    provides=[],
    install_requires=[
        "Click==7.1.2",
        "jinja2==2.11.2",
        "pyaml==20.4.0",
        "cfn-lint==0.44.1",
        # "pytest==6.1.2",
        # "coverage==5.3",
        # "moto==1.3.16",
        "google-cloud-storage==1.35.0",
        # "pytest-cov==2.10.1",
    ],
    namespace_packages=[],
    packages=find_packages(),
    package_data = {
        'templates':['*']
    },
    entry_points="""
        [console_scripts]
        freeldep=freeldep.main:cli
    """,
    zip_safe=False,
)



import json
import os

import pytest
import yaml

from freeldep.modules.utils import delete_deployer
from freeldep.modules.utils import execute_script
from freeldep.modules.utils import file_exists
from freeldep.modules.utils import get_workflow_id
from freeldep.modules.utils import list_files
from freeldep.modules.utils import load_deployer
from freeldep.modules.utils import mkdir
from freeldep.modules.utils import read_json
from freeldep.modules.utils import read_yaml
from freeldep.modules.utils import save_deployer
from freeldep.modules.utils import save_json
from freeldep.modules.utils import shorten_expr
from freeldep.modules.utils import valid_expr

TEMP_FOLDER = "/tmp/"


def test_valid_expr():
    assert valid_expr("test", "test", r"^test$") is None
    with pytest.raises(ValueError):
        valid_expr("test", "testtest", r"^test$")


def test_shorten_expr():
    assert shorten_expr("12345", 10) == "12345"
    assert shorten_expr("12345", 2) == "12"


def test_mkdir():
    directory = TEMP_FOLDER + "pytest/subg"
    assert os.path.isdir(directory) is False
    mkdir(directory)
    assert os.path.isdir(directory) is True
    os.removedirs(directory)


def test_file_exists():
    filename = TEMP_FOLDER + "pytest.txt"
    assert file_exists(filename) is False
    with open(filename, "w+") as f:
        json.dump([], f)
    assert file_exists(filename) is True
    os.remove(filename)


def test_list_files():
    FOLDER = "./tests/data/"
    files = list_files(FOLDER)
    for f in files:
        print(f)
        assert os.path.isfile(FOLDER + f)
    assert len(files) == 3


def test_save_json():
    filename = TEMP_FOLDER + "test.json"
    data = {"test": True}
    if file_exists(filename):
        os.remove(filename)
    assert file_exists(filename) is False
    save_json(filename, data)
    assert file_exists(filename)
    data = read_json(filename)
    assert data["test"] is True


def test_read_yaml():
    filename = TEMP_FOLDER + "test.yaml"
    data = {"test": True}
    with open(filename, "w+") as f:
        yaml.dump(data, f)
    assert file_exists(filename)
    data = read_yaml(filename)
    assert data["test"] is True


def test_manage_deployer(deployer, config, configfile):
    home_folder = configfile["cli"].get("home_folder")
    test_deployer = home_folder + "/test.json"
    assert file_exists(test_deployer) is False
    save_deployer(deployer, config)
    assert file_exists(test_deployer) is True
    saved_deployer = load_deployer(config, "test")
    for key in saved_deployer:
        assert saved_deployer[key] == deployer[key]
    delete_deployer(config, "test")
    assert file_exists(test_deployer) is False


def test_load_deployer(deployer, config, configfile):
    home_folder = configfile["cli"].get("home_folder")
    test_deployer = home_folder + "/test.json"
    assert file_exists(test_deployer) is False
    save_deployer(deployer, config)
    assert file_exists(test_deployer) is True
    saved_deployer = load_deployer(config, "test")
    for key in saved_deployer:
        assert saved_deployer[key] == deployer[key]
    delete_deployer(config, "test")
    assert file_exists(test_deployer) is False


def test_load_missing_deployer(deployer, config, configfile):
    saved_deployer = load_deployer(config, "test")
    assert saved_deployer is None


def test_execute_script():
    filename = TEMP_FOLDER + "test.sh"
    with open(filename, "w+") as f:
        f.write("ls -l")
    assert execute_script(filename)
    os.remove(filename)
    filename = TEMP_FOLDER + "test.sh"
    with open(filename, "w+") as f:
        f.write("thisisgoingtofail")
    assert execute_script(filename) is False
    os.remove(filename)


def test_get_workflow_id(deployer, config):
    workflow_name = "test-workflow"
    expected_aws_arn = (
        f"arn:aws:states:region:123456789012:stateMachine:{workflow_name}"
    )
    assert get_workflow_id("test-workflow", config) == expected_aws_arn



import os
import uuid

import pytest

from freeldep.cloud.aws.validate import CfnStackValidation


def test_file_exists():
    filename = "/tmp/" + uuid.uuid1().hex + ".test"
    assert CfnStackValidation._file_exists(filename) is False
    with open(filename, "w+") as f:
        f.write("hello")
    assert CfnStackValidation._file_exists(filename) is True
    os.remove(filename)


def test_validate_lambda():
    filename = "/tmp/" + uuid.uuid1().hex + ".test"
    with open(filename, "w+") as f:
        f.write("hello")
    config = {
        "name": "test",
        "location": filename,
        "template-attribute": "asdasd",
        "bucket": "sfsdfs",
    }
    with pytest.raises(ValueError):
        CfnStackValidation._validate_lambda_config([config])
    os.remove(filename)
    config["location"] = "/tmp/"
    assert CfnStackValidation._validate_lambda_config([config]) is None
    config_failed = config.copy()
    del config_failed["name"]
    with pytest.raises(KeyError):
        CfnStackValidation._validate_lambda_config([config_failed])
    config_failed = config.copy()
    del config_failed["location"]
    with pytest.raises(KeyError):
        CfnStackValidation._validate_lambda_config([config_failed])


def test_config(template):
    assert CfnStackValidation.validate_config(template) is None
    template_test = template.copy()
    del template_test["aws"]
    with pytest.raises(KeyError):
        CfnStackValidation.validate_config(template_test)
    template_test = template.copy()
    del template_test["template"]
    with pytest.raises(KeyError):
        CfnStackValidation.validate_config(template_test)
    template_test = template.copy()
    del template_test["location"]
    with pytest.raises(KeyError):
        CfnStackValidation.validate_config(template_test)
    template_test = template.copy()
    template_test["location"] = "dsdsdds"
    with pytest.raises(FileNotFoundError):
        CfnStackValidation.validate_config(template_test)


def test_validate_template():
    config = {"name": "test", "parameters": ""}
    with pytest.raises(ValueError):
        CfnStackValidation._validate_template_config([config])
    with pytest.raises(ValueError):
        CfnStackValidation._validate_template_config(config)
    config = {"name": "test", "parameters": {}}
    assert CfnStackValidation._validate_template_config(config) is None


def test_validate_aws():
    config = {"region": "test", "account-id": "111111111111"}
    assert CfnStackValidation._validate_aws_config(config) is None
    config["region"] = 22
    with pytest.raises(ValueError):
        CfnStackValidation._validate_aws_config(config)
    with pytest.raises(ValueError):
        CfnStackValidation._validate_aws_config([config])
    config = {"region": "test", "account-id": "222"}
    with pytest.raises(ValueError):
        CfnStackValidation._validate_aws_config(config)
    config = {"name": "test", "parameters": {}}
    with pytest.raises(KeyError):
        CfnStackValidation._validate_aws_config(config)
    config = {"region": "test", "parameters": {}}
    with pytest.raises(KeyError):
        CfnStackValidation._validate_aws_config(config)



import json
import logging
import os
import time
import uuid
from datetime import datetime

import boto3
import botocore
import yaml

from freeldep.cloud.aws.status import StackStatus
from freeldep.cloud.aws.validate import CfnStackValidation
from freeldep.cloud.stack import CloudStack


class CfnStack(CloudStack):
    """CFN Stack: Read template, Inject parameters, Build lambdas and upload"""

    def __init__(self, config, deployer):
        super().__init__(config, deployer)
        CfnStackValidation.validate_config(config)

    def validate_template(self):
        """Validate CFN template with CFNLINT"""
        if self.render_template is None:
            self.render_template()
        CfnStackValidation.validate_stack(self.rendered_template)

    def deploy(self, wait=False, dryrun=False, output_folder=None):
        """Package Lambda functions, deploy CFN template and wait if needed"""
        self.package_lambdas(dryrun=dryrun, output_folder=output_folder)
        self.create_stack(dryrun=dryrun, output_folder=output_folder)
        if wait and not dryrun:
            self.wait_completion(wait)

    def package_lambdas(self, dryrun=False, output_folder=None, s3_client=None):
        """Package Lambda function into a zip file and upload to S3"""
        s3 = self._boto_client("s3", s3_client)
        bucket = self.deployer["artifact"]
        for lambd in self.config["functions"]:
            filename = (
                lambd["filename"]
                if output_folder is None
                else output_folder + lambd["filename"]
            )
            self._compress_folder(lambd["location"], filename)
            key = self.config["template"][lambd["template-attribute"]]
            if not dryrun:
                self._upload_object(s3, bucket, key, open(filename, "rb").read())
                os.remove(filename)

    def create_stack(
        self, dryrun=False, output_folder=None, cloudformation_client=None
    ):
        """Deploy CFN template"""
        cfn_parameters = self._format_template_parameters(
            self.config["template"].get("parameters", {})
        )
        cloudformation = self._boto_client("cloudformation", cloudformation_client)
        cfn_name = self.config["template"]["name"]
        if dryrun:
            filename = cfn_name if output_folder is None else output_folder + cfn_name
            self._write_txt(self.rendered_template, filename + ".yaml")
            self._write_yaml(cfn_parameters, filename + ".config.yaml")
            return
        try:
            _ = cloudformation.describe_stacks(StackName=cfn_name)
            cloudformation.update_stack(
                StackName=cfn_name,
                TemplateBody=self.rendered_template,
                Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
                Parameters=cfn_parameters,
            )
            return True
        except botocore.exceptions.ClientError as error:
            if (
                error.response["Error"]["Code"] == "ValidationError"
                and error.response["Error"]["Message"]
                == "No updates are to be performed."
            ):
                logging.error(error.response["Error"]["Message"])
                return False
            if (
                error.response["Error"]["Code"] == "ValidationError"
                and error.response["Error"]["Message"]
                == f"Stack with id {cfn_name} does not exist"
            ):
                cloudformation.create_stack(
                    StackName=cfn_name,
                    TemplateBody=self.rendered_template,
                    Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
                    Parameters=cfn_parameters,
                )
                return True
            raise

    def wait_completion(self, wait, cloudformation_client=None):
        """Wait for a stack to be deployed"""
        waiting = wait
        cloudformation = self._boto_client("cloudformation", cloudformation_client)
        while waiting:
            stack = cloudformation.describe_stacks(
                StackName=self.config["template"]["name"]
            )
            status = stack["Stacks"][0]["StackStatus"]
            if StackStatus.success(status):
                logging.info(f"Status: {status}")
                waiting = False
            elif StackStatus.failed(status):
                raise RuntimeError(f"Status: {status}")
            time.sleep(10)
            logging.info(f"Checking status of stack {self.config['template']['name']}")

    def cleanup(
        self, wait, dryrun=False, output_folder=None, cloudformation_client=None
    ):
        """Delete existing stack"""
        waiting = wait
        cloudformation = self._boto_client("cloudformation", cloudformation_client)
        stack_name = self.config["template"]["name"]
        if not self._stack_exists(cloudformation, stack_name):
            return False, "Stack does not exist"
        if dryrun:
            return True
        cloudformation.delete_stack(StackName=stack_name)
        while waiting:
            try:
                stack = cloudformation.describe_stacks(StackName=stack_name)
                status = stack["Stacks"][0]["StackStatus"]
                if StackStatus.success(status):
                    logging.info(f"Status: {status}")
                    waiting = False
                elif StackStatus.failed(status):
                    raise RuntimeError(f"Status: {status}")
                time.sleep(10)
                logging.info(f"Checking status of stack {stack_name}")
            except Exception:
                return True

    def trigger_deployment(self, dryrun=False, output_folder=None):
        if (
            "artifact" not in self.deployer
            or "deployment-workflow-id" not in self.deployer
        ):
            raise RuntimeError("No deployment workflow")
        if not dryrun:
            template_location = self._upload_template()
            sfn = self._boto_client("stepfunctions")
            execution_id = self.config["template"]["name"] + "-" + uuid.uuid1().hex
            sfn.start_execution(
                stateMachineArn=self.deployer["deployment-workflow-id"],
                name=execution_id,
                input=json.dumps(
                    {
                        "TemplateName": self.config["template"]["name"],
                        "DeploymentTimestamp": str(int(time.time())),
                        "TemplateLocation": template_location,
                        "Action": self.config.get(
                            "action", "create_or_update_stack"
                        ).upper(),
                        "Test": self._configure_tests(template_location),
                        "Valid": True,
                        "KeepFailed": self.config.get("cleanup", False),
                        "NotifyOnFailure": self.config.get("subscription", ""),
                        "Parameters": {
                            "".join([x.title() for x in key.split("-")]): val
                            for key, val in self.config["template"]
                            .get("parameters", {})
                            .items()
                        },
                    }
                ),
            )

    def _format_template_parameters(self, parameters):
        """Format parmeters camel case"""
        return [
            {
                "ParameterKey": "".join([x.title() for x in key.split("-")]),
                "ParameterValue": val,
            }
            for key, val in parameters.items()
        ]

    def _format_config(self):
        conf = {
            "AwsRegion": self.config["aws"]["region"],
            "AwsAccountId": self.config["aws"]["account-id"],
        }
        if "deployment-role" in self.config["aws"]:
            conf["AwsDeploymentRole"] = self.config["aws"]["deployment-role"]
        for param, val in self.config["template"].items():
            conf["".join(x.title() for x in param.split("-"))] = val
        if "parameters" in self.config["template"]:
            for param, val in self.config["template"]["parameters"].items():
                conf["".join(x.title() for x in param.split("-"))] = val
        return conf

    def _boto_client(self, service, client=None):
        """Assume deployment role if any, else get client"""
        if os.environ.get("PYTEST", ""):
            if os.environ.get("AWS_PROFILE", ""):
                del os.environ["AWS_PROFILE"]
            return boto3.client(
                service, aws_secret_access_key="NONE", aws_access_key_id="NONE"
            )
        if client:
            return client
        if self.config["aws"].get("deployment-role", ""):
            sts = boto3.client("sts", region_name=self.config["aws"]["region"])
            creds = sts.assume_role(
                RoleArn=self.config["aws"]["deployment-role"], RoleSessionName="tmp"
            )
            return boto3.client(
                service,
                aws_secret_access_key=creds["Credentials"]["SecretAccessKey"],
                aws_access_key_id=creds["Credentials"]["AccessKeyId"],
                aws_session_token=creds["Credentials"]["SessionToken"],
            )
        return boto3.client(service, region_name="ap-southeast-1")

    def _upload_object(self, s3, bucket, key, content):
        """Upload object to S3"""
        s3.put_object(Bucket=bucket, Key=key, Body=content)

    def _upload_template(self):
        """Upload a project CFN template to S3"""
        s3 = self._boto_client("s3")
        name = datetime.now().strftime("%Y%m%d_%H%M%S" + ".yaml")
        key = f"templates/{self.deployer['project']}/{self.config['template']['name']}/{name}"
        s3.put_object(
            Bucket=self.deployer["artifact"], Key=key, Body=self.rendered_template
        )
        return f"s3://{self.deployer['artifact']}/{key}"

    def _configure_tests(self, template_location):
        """Upload taskcat config file to artifact bucket"""
        if "test" not in self.config or not self._file_exists(self.config["test"]):
            return ""
        taskcat = self._read_yaml(self.config["test"])
        taskcat["project"]["s3_bucket"] = self.deployer["artifact"]
        taskcat["project"]["name"] = f"{self.deployer['project']}-tests"
        taskcat["tests"]["default"]["template"], key = template_location.split("/")[-1]
        s3 = self._boto_client("s3")
        s3.put_object(
            Bucket=self.deployer["artifact"], Key=key, Body=yaml.dump(taskcat)
        )
        return f"s3://{self.deployer['artifact']}/{key}"

    def _format_test_template_location(self, location):
        template = location.split("/")[-1]
        key = "/".join(location.split("/")[3:-1]) + "/_tests/" + location.split("/")[-1]
        return template, key

    def _file_exists(self, file_path):
        """Check if a file exists"""
        return os.path.exists(os.path.expanduser(file_path))

    def _read_yaml(self, filename):
        """Read a YAML file"""
        with open(filename, "r") as stream:
            return yaml.safe_load(stream)

    def _stack_exists(self, cloudformation, stack_name):
        """Check if a CFN stack exists or not"""
        try:
            cloudformation.describe_stacks(StackName=stack_name)
            return True
        except Exception:
            return False



import os

from click.testing import CliRunner

from freeldep.main import cli


CONFIG_SAMPLE = "./tests/data/config.ini"


def test_create_stack_missing_deployer(deployer, config):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "deploy",
            "core",
            "--deployer",
            "missing",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "service",
            "--deployer",
            "missing",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0


def test_deploy_project_errors():
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "deploy",
            "project",
            "--deployer",
            "testt",
            "--project",
            "missing",
            "--stack-file",
            "examples/config.yaml",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "project",
            "--project",
            "missing",
            "--stack-file",
            "examples/config.yaml",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0


def test_deploy_project_without_core():
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "deployer",
            "testwithoutcore",
            "--cloud",
            "AWS",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "deploy",
            "project",
            "--deployer",
            "testwithoutcore",
            "--project",
            "test",
            "--stack-file",
            "examples/config.yaml",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "testwithoutcore", "--confirm"],
    )
    assert result.exit_code == 0


def test_create_others_with_errors(deployer, config, aws_credentials):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "deployer",
            "testingerrors",
            "--cloud",
            "AWS",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "subscription",
            "test",
            "--emails",
            "test@gmial.com",
            "--deployer",
            "doesntexist",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "subscription",
            "test",
            "--emails",
            "dfsdfdssdf",
            "--deployer",
            "testingerrors",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "project",
            "testproj",
            "--deployer",
            "sdfsdfsdfds",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "project",
            "testproj",
            "--deployer",
            "testingerrors",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "testingerrors", "--confirm"],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "deployer",
            "testingerrorstwo",
            "--cloud",
            "GCP",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "subscription",
            "test",
            "--emails",
            "dfsdfdssdf",
            "--deployer",
            "testingerrorstwo",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "testingerrorstwo", "--confirm"],
    )
    assert result.exit_code == 0


def test_create_repository_error(deployer, config, aws_credentials):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "repository",
            "--deployer",
            "doesntwsief",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code != 0



import os
import uuid
import zipfile
from datetime import datetime

import yaml
from jinja2 import Template  # pylint: disable=import-error

class CloudStack(object):
    def __init__(self, config, deployer):
        self.config = config
        self.deployer = deployer
        self.template = None
        self.rendered_template = None

        if "functions" not in self.config:
            self.config["functions"] = []

        if "gcloud" not in self.config:
            self.config["gcloud"] = []

    def read_template(self):
        """Import template"""
        if type(self.config['location']) == str:
            with open(self.config["location"], "r") as stream:
                self.template = stream.read()
        else:
            self.template = self.config["location"].read()



    def prepare_packages(self):
        """Prepare Function packages and update stack parameters"""
        for idx, function in enumerate(self.config["functions"]):
            uname = datetime.now().strftime(
                function["name"] + "_" + "%Y%m%d_%H%M_" + uuid.uuid1().hex + ".zip"
            )
            self.config["template"][function["template-attribute"]] = (
                self.config["template"][function["template-attribute"]] + uname
            )
            self.config["functions"][idx]["filename"] = uname
            self.config["functions"][idx]["key"] = self.config["template"][
                function["template-attribute"]
            ]

    def render_template(self):
        """Inject Jinja parameters into template"""
        template_properties = self._format_config()
        if self.template is None:
            self.read_template()
        tm = Template(self.template)
        self.rendered_template = tm.render(**template_properties)

    def _compress_folder(self, location, filename):
        """Compress a folder"""
        rootlen = len(location)
        zipf = zipfile.ZipFile(filename, "w", zipfile.ZIP_DEFLATED)
        for root, _, files in os.walk(location):
            for f in files:
                fn = os.path.join(root, f)
                zipf.write(fn, fn[rootlen:])
        zipf.close()
        return filename

    def _write_yaml(self, obj, filename):
        """Write a YAML file"""
        with open(filename, "w+") as stream:
            yaml.dump(obj, stream)

    def _write_txt(self, obj, filename):
        """Write a TXT file"""
        with open(filename, "w+") as stream:
            stream.write(obj)

    def validate_template(self):
        raise NotImplementedError("cli.cloud.stack.validate_template()")

    def deploy(self, wait=False, dryrun=False, output_folder=None):
        raise NotImplementedError("cli.cloud.stack.deploy()")

    def package_functions(self, dryrun=False, output_folder=None, gs_client=None):
        raise NotImplementedError("cli.cloud.stack.package_functions()")

    def deploy_template(self, dryrun=False, output_folder=None):
        raise NotImplementedError("cli.cloud.stack.deploy_template()")

    def cleanup(self, wait=True, dryrun=False, output_folder=None):
        raise NotImplementedError("cli.cloud.stack.cleanup()")

    def trigger_deployment(self, dryrun=False, output_folder=None):
        raise NotImplementedError("cli.cloud.stack.trigger_deployment()")

    def _format_config(self):
        raise NotImplementedError("cli.cloud.stack._format_config()")



# from cli.cloud.aws.stack import CfnStack
# def read_template(self):
#     """Read CFN template as text file"""
#     with open(self.config['location'], "r") as stream:
#         self.template = stream.read()
# def prepare_packages(self):
#     """Prepare lambda packages and update stack parameters"""
#     for idx, lambd in enumerate(self.config['lambdas']):
#         uname = datetime.now().strftime(
#             lambd["name"] + "_" + "%Y%m%d_%H%M_" + uuid.uuid1().hex + ".zip"
#         )
#         self.config['template'][lambd["template-attribute"]] = (
#             self.config['template'][lambd["template-attribute"]] + uname
#         )
#         self.config['lambdas'][idx]["filename"] = uname
#         self.config['lambdas'][idx]["key"] = self.config['template'][lambd["template-attribute"]]
# def _format_config(self):
#     conf = {
#         "AwsRegion": self.config["aws"]["region"],
#         "AwsAccountId": self.config["aws"]["account-id"],
#     }
#     if "deployment-role" in self.config["aws"]:
#         conf["AwsDeploymentRole"] = self.config["aws"]["deployment-role"]
#     for param, val in self.config['template'].items():
#         conf["".join(x.title() for x in param.split("-"))] = val
#     if "parameters" in self.config['template']:
#         for param, val in self.config['template']["parameters"].items():
#             conf["".join(x.title() for x in param.split("-"))] = val
#     return conf
# def _format_cfn_parameters(self, parameters):
#     """Format parmeters camel case"""
#     return [
#         {
#             "ParameterKey": "".join([x.title() for x in key.split("-")]),
#             "ParameterValue": val,
#         }
#         for key, val in parameters.items()
#     ]
# def _compress_folder(self, location, filename):
#     """Compress a folder"""
#     rootlen = len(location)
#     zipf = zipfile.ZipFile(filename, "w", zipfile.ZIP_DEFLATED)
#     for root, _, files in os.walk(location):
#         for f in files:
#             fn = os.path.join(root, f)
#             zipf.write(fn, fn[rootlen:])
#     zipf.close()
#     return filename
# def _format_test_template_location(self, location):
#     template = location.split("/")[-1]
#     key = (
#         "/".join(location.split("/")[3:-1])
#         + "/_tests/"
#         + location.split("/")[-1]
#     )
#     return template, key
# def _file_exists(self, file_path):
#     """Check if a file exists"""
#     return os.path.exists(os.path.expanduser(file_path))
# def _read_yaml(self, filename):
#     """Read a YAML file"""
#     with open(filename, "r") as stream:
#         return yaml.safe_load(stream)
# def _write_yaml(self, obj, filename):
#     """Write a YAML file"""
#     with open(filename, "w+") as stream:
#         yaml.dump(obj, stream)



import json
import os
from subprocess import call

from click.testing import CliRunner

from freeldep.main import cli
from freeldep.modules.utils import delete_deployer
from freeldep.modules.utils import load_deployer
from freeldep.modules.utils import save_deployer


CONFIG_SAMPLE = "./tests/data/config.ini"


def test_show_configs(deployer, config):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    runner = CliRunner()
    result = runner.invoke(cli, ["show", "configs"])
    assert result.exit_code == 0
    save_deployer(deployer, config)
    result = runner.invoke(cli, ["show", "configs"])
    assert "test" in result.output.strip()
    delete_deployer(config, "test")


def test_show_config(deployer, config):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    delete_deployer(config, "test")
    runner = CliRunner()
    result = runner.invoke(cli, ["show", "config", "test"])
    assert result.exit_code != 0
    save_deployer(deployer, config)
    result = runner.invoke(cli, ["show", "config", "test"])
    assert result.exit_code == 0
    assert json.loads(result.output)["name"] == "test"
    delete_deployer(config, "test")


def test_create_stack(deployer, config):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "deployer",
            "testt",
            "--cloud",
            "AWS",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-initialization-stack.config.yaml")
    assert os.path.isfile("out/testt-deployer-initialization-stack.yaml")
    returncall = call(
        "cfn-lint out/testt-deployer-initialization-stack.yaml", shell=True
    )
    assert returncall == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "core",
            "--deployer",
            "testt",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-core-stack.config.yaml")
    assert os.path.isfile("out/testt-deployer-core-stack.config.yaml")
    returncall = call("cfn-lint out/testt-deployer-core-stack.yaml", shell=True)
    assert returncall == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "service",
            "--deployer",
            "testt",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-service-stack.config.yaml")
    assert os.path.isfile("out/testt-deployer-service-stack.yaml")
    returncall = call("cfn-lint out/testt-deployer-service-stack.yaml", shell=True)
    assert returncall == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "project",
            "--deployer",
            "testt",
            "--project",
            "mytest",
            "--stack-file",
            "examples/config.yaml",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    os.environ[
        "CODEBUILD_SOURCE_REPO_URL"
    ] = "https://git-codecommit.ap-southeast-1.amazonaws.com/v1/repos/de-testproj"
    os.environ["ARTIFACTS_BUCKET"] = "testbucket"
    os.environ["DEPLOYER_STATE_MACHINE_ARN"] = "mytestworkiflow"
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "deploy",
            "project",
            "--deployer",
            "testt",
            "--project",
            "de-testproj",
            "--stack-file",
            "examples/config.yaml",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0


def test_create_others(deployer, config, aws_credentials):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "subscription",
            "test",
            "--emails",
            "test@gmial.com",
            "--deployer",
            "testt",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-test-subscription.config.yaml")
    assert os.path.isfile("out/testt-deployer-test-subscription.yaml")
    returncall = call("cfn-lint out/testt-deployer-test-subscription.yaml", shell=True)
    assert returncall == 0
    depl = load_deployer(config, "testt")
    assert "testt-deployer-test" in depl["subscriptions"]
    os.environ["AWS_PROFILE"] = "pytest"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "create",
            "project",
            "testproj",
            "--deployer",
            "testt",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert returncall == 0
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-project-testproj.config.yaml")
    assert os.path.isfile("out/testt-deployer-project-testproj.yaml")
    returncall = call("cfn-lint out/testt-deployer-project-testproj.yaml", shell=True)
    assert returncall == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        [
            "create",
            "repository",
            "--deployer",
            "testt",
            "--dryrun",
            "--output-location",
            "out/",
        ],
    )
    assert result.exit_code == 0
    assert os.path.isfile("out/testt-deployer-repository-stack.config.yaml")
    assert os.path.isfile("out/testt-deployer-repository-stack.yaml")
    returncall = call("cfn-lint out/testt-deployer-repository-stack.yaml", shell=True)
    assert returncall == 0


def test_cleanup_others(deployer, config):
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "cleanup",
            "subscription",
            "testt-deployer-test",
            "--deployer",
            "testt",
            "--dryrun",
        ],
    )
    assert result.exit_code == 0
    depl = load_deployer(config, "testt")
    assert "testt-deployer-test" not in depl["subscriptions"]
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "subscription", "asdasdsda", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "subscription", "asdasdsda", "--deployer", "asdasdasd", "--dryrun"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "project", "testproj", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code == 0
    depl = load_deployer(config, "testt")
    assert "project" not in depl or "testproj" not in depl["project"]
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "project", "asdasdasd", "--deployer", "jhghgjghj", "--dryrun"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "repository", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code == 0
    depl = load_deployer(config, "testt")
    print(depl)
    assert "repository" not in depl
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "repository", "--deployer", "asdasdasd", "--dryrun"],
    )
    assert result.exit_code != 0


def test_cleanup():
    os.environ["FREELDEP_CONFIG"] = CONFIG_SAMPLE
    os.environ["AWS_PROFILE"] = "pytest"
    os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-1"
    os.environ["PYTEST"] = "true"
    runner = CliRunner()
    os.environ["CODEBUILD_SOURCE_REPO_URL"] = ""
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "core", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "service", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "dsadasd", "--confirm"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "testt"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "deployer", "testt", "--confirm"],
    )
    assert result.exit_code == 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "core", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code != 0
    os.environ["AWS_PROFILE"] = "pytest"
    result = runner.invoke(
        cli,
        ["cleanup", "service", "--deployer", "testt", "--dryrun"],
    )
    assert result.exit_code != 0



import os

import click
import pytest

from freeldep.config import ConfigParser


def test_config_error():
    os.environ["FREELDEP_CONFIG"] = ""
    os.environ["FREELDEP_HOME"] = ""
    with pytest.raises(click.UsageError):
        ConfigParser("")


def test_config_filename():
    os.environ["FREELDEP_CONFIG"] = ""
    os.environ["FREELDEP_HOME"] = ""
    ConfigParser("./tests/data/config.ini")


def test_config_env():
    os.environ["FREELDEP_CONFIG"] = ""
    os.environ["FREELDEP_HOME"] = "./freeldep/"
    ConfigParser("")
    os.environ["FREELDEP_HOME"] = "./freeldep"
    ConfigParser("")


def test_config_env_2():
    os.environ["FREELDEP_CONFIG"] = "./tests/data/config.ini"
    os.environ["FREELDEP_HOME"] = ""
    ConfigParser("")


def test_config_get_en():
    os.environ["FREELDEP_CONFIG"] = ""
    os.environ["FREELDEP_HOME"] = ""
    config = ConfigParser("./tests/data/config.ini")
    assert config.get("cli", "name", "") == "testcli"
    assert config.get("cli", "asdasd", "default") == "default"
    assert config.get("assdasd", "asdasd", "1111") == "1111"
    assert config.get("aws", "region", "1111") == "region"
